@@ -263,7 +263,9 @@ Q_SIGNALS:
     void parityChanged(QSerialPort::Parity parity);
     void stopBitsChanged(QSerialPort::StopBits stopBits);
     void flowControlChanged(QSerialPort::FlowControl flowControl);
+#if QT_DEPRECATED_SINCE(5, 2)
     void dataErrorPolicyChanged(QSerialPort::DataErrorPolicy policy);
+#endif
     void dataTerminalReadyChanged(bool set);
     void requestToSendChanged(bool set);
     void error(QSerialPort::SerialPortError serialPortError);
@@ -282,9 +284,10 @@ private:
     Q_DISABLE_COPY(QSerialPort)
 
 #if defined (Q_OS_WIN32) || defined(Q_OS_WIN64)
-    Q_PRIVATE_SLOT(d_func(), void _q_completeAsyncCommunication())
-    Q_PRIVATE_SLOT(d_func(), void _q_completeAsyncRead())
-    Q_PRIVATE_SLOT(d_func(), void _q_completeAsyncWrite())
+    Q_PRIVATE_SLOT(d_func(), bool _q_completeAsyncCommunication())
+    Q_PRIVATE_SLOT(d_func(), bool _q_completeAsyncRead())
+    Q_PRIVATE_SLOT(d_func(), bool _q_completeAsyncWrite())
+    Q_PRIVATE_SLOT(d_func(), bool _q_startAsyncWrite())
 #endif
 };
 
