@@ -204,39 +204,11 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
         return false;
     }
 
-#ifdef TIOCEXCL
-    if (::ioctl(descriptor, TIOCEXCL) == -1)
-        q->setError(decodeSystemError());
-#endif
-
-    if (::tcgetattr(descriptor, &restoredTermios) == -1) {
-        q->setError(decodeSystemError());
+    if (!initialize(mode)) {
+        qt_safe_close(descriptor);
         return false;
     }
 
-    currentTermios = restoredTermios;
-#ifdef Q_OS_SOLARIS
-    currentTermios.c_iflag &= ~(IMAXBEL|IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
-    currentTermios.c_oflag &= ~OPOST;
-    currentTermios.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
-    currentTermios.c_cflag &= ~(CSIZE|PARENB);
-    currentTermios.c_cflag |= CS8;
-#else
-    ::cfmakeraw(&currentTermios);
-#endif
-    currentTermios.c_cflag |= CLOCAL;
-    currentTermios.c_cc[VTIME] = 0;
-    currentTermios.c_cc[VMIN] = 0;
-
-    if (mode & QIODevice::ReadOnly)
-        currentTermios.c_cflag |= CREAD;
-
-    if (!updateTermios())
-        return false;
-
-    if ((flags & O_WRONLY) == 0)
-        setReadNotificationEnabled(true);
-
     lockFileScopedPointer.swap(newLockFileScopedPointer);
 
     return true;
@@ -365,12 +337,7 @@ bool QSerialPortPrivate::setRequestToSend(bool set)
 
 bool QSerialPortPrivate::flush()
 {
-    return completeAsyncWrite()
-#ifndef Q_OS_ANDROID
-            && (::tcdrain(descriptor) != -1);
-#else
-            && (::ioctl(descriptor, TCSBRK, 1) != -1);
-#endif
+    return completeAsyncWrite();
 }
 
 bool QSerialPortPrivate::clear(QSerialPort::Directions directions)
@@ -435,14 +402,11 @@ bool QSerialPortPrivate::waitForReadyRead(int msecs)
             return false;
         }
 
-        if (readyToRead) {
-            if (readNotification())
-                return true;
-        }
-
-        if (readyToWrite)
-            completeAsyncWrite();
+        if (readyToRead)
+            return readNotification();
 
+        if (readyToWrite && !completeAsyncWrite())
+            return false;
     } while (msecs == -1 || timeoutValue(msecs, stopWatch.elapsed()) > 0);
     return false;
 }
@@ -872,6 +836,46 @@ bool QSerialPortPrivate::completeAsyncWrite()
     return startAsyncWrite();
 }
 
+inline bool QSerialPortPrivate::initialize(QIODevice::OpenMode mode)
+{
+    Q_Q(QSerialPort);
+
+#ifdef TIOCEXCL
+    if (::ioctl(descriptor, TIOCEXCL) == -1)
+        q->setError(decodeSystemError());
+#endif
+
+    if (::tcgetattr(descriptor, &restoredTermios) == -1) {
+        q->setError(decodeSystemError());
+        return false;
+    }
+
+    currentTermios = restoredTermios;
+#ifdef Q_OS_SOLARIS
+    currentTermios.c_iflag &= ~(IMAXBEL|IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
+    currentTermios.c_oflag &= ~OPOST;
+    currentTermios.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
+    currentTermios.c_cflag &= ~(CSIZE|PARENB);
+    currentTermios.c_cflag |= CS8;
+#else
+    ::cfmakeraw(&currentTermios);
+#endif
+    currentTermios.c_cflag |= CLOCAL;
+    currentTermios.c_cc[VTIME] = 0;
+    currentTermios.c_cc[VMIN] = 0;
+
+    if (mode & QIODevice::ReadOnly)
+        currentTermios.c_cflag |= CREAD;
+
+    if (!updateTermios())
+        return false;
+
+    if (mode & QIODevice::ReadOnly)
+        setReadNotificationEnabled(true);
+
+    return true;
+}
+
 bool QSerialPortPrivate::updateTermios()
 {
     Q_Q(QSerialPort);
