@@ -43,6 +43,17 @@
 #ifndef QSERIALPORT_UNIX_P_H
 #define QSERIALPORT_UNIX_P_H
 
+//
+//  W A R N I N G
+//  -------------
+//
+// This file is not part of the Qt API.  It exists purely as an
+// implementation detail.  This header file may change from version to
+// version without notice, or even be removed.
+//
+// We mean it.
+//
+
 #include "qserialport_p.h"
 
 #include <QtCore/qlockfile.h>
@@ -124,7 +135,6 @@ public:
     bool readNotification();
     bool startAsyncWrite();
     bool completeAsyncWrite();
-    void exceptionNotification();
 
     static QString portNameToSystemLocation(const QString &port);
     static QString portNameFromSystemLocation(const QString &location);
@@ -136,16 +146,10 @@ public:
 
     struct termios currentTermios;
     struct termios restoredTermios;
-#ifdef Q_OS_LINUX
-    struct serial_struct currentSerialInfo;
-    struct serial_struct restoredSerialInfo;
-#endif
     int descriptor;
-    bool isCustomBaudRateSupported;
 
     QSocketNotifier *readNotifier;
     QSocketNotifier *writeNotifier;
-    QSocketNotifier *exceptionNotifier;
 
     bool readPortNotifierCalled;
     bool readPortNotifierState;
@@ -160,16 +164,21 @@ public:
     QScopedPointer<QLockFile> lockFileScopedPointer;
 
 private:
+    bool initialize(QIODevice::OpenMode mode);
     bool updateTermios();
 
+    QSerialPort::SerialPortError setBaudRate_helper(qint32 baudRate,
+            QSerialPort::Directions directions);
+    QSerialPort::SerialPortError setCustomBaudRate(qint32 baudRate,
+            QSerialPort::Directions directions);
+    QSerialPort::SerialPortError setStandardBaudRate(qint32 baudRate,
+            QSerialPort::Directions directions);
     QSerialPort::SerialPortError decodeSystemError() const;
 
     bool isReadNotificationEnabled() const;
     void setReadNotificationEnabled(bool enable);
     bool isWriteNotificationEnabled() const;
     void setWriteNotificationEnabled(bool enable);
-    bool isExceptionNotificationEnabled() const;
-    void setExceptionNotificationEnabled(bool enable);
 
     bool waitForReadOrWrite(bool *selectForRead, bool *selectForWrite,
                             bool checkRead, bool checkWrite,
