@@ -44,12 +44,9 @@
 #include "qserialport_win_p.h"
 
 #include <QtCore/qcoreevent.h>
-
-#ifndef Q_OS_WINCE
 #include <QtCore/qelapsedtimer.h>
 #include <QtCore/qvector.h>
-#endif
-
+#include <QtCore/qtimer.h>
 #include <QtCore/qwineventnotifier.h>
 #include <algorithm>
 
@@ -86,8 +83,6 @@
 
 QT_BEGIN_NAMESPACE
 
-#ifndef Q_OS_WINCE
-
 static void initializeOverlappedStructure(OVERLAPPED &overlapped)
 {
     overlapped.Internal = 0;
@@ -106,6 +101,7 @@ QSerialPortPrivate::QSerialPortPrivate(QSerialPort *q)
     , communicationNotifier(new QWinEventNotifier(q))
     , readCompletionNotifier(new QWinEventNotifier(q))
     , writeCompletionNotifier(new QWinEventNotifier(q))
+    , startAsyncWriteTimer(0)
     , originalEventMask(0)
     , triggeredEventMask(0)
 {
@@ -159,56 +155,11 @@ bool QSerialPortPrivate::open(QIODevice::OpenMode mode)
         return false;
     }
 
-    ::ZeroMemory(&restoredDcb, sizeof(restoredDcb));
-    restoredDcb.DCBlength = sizeof(restoredDcb);
+    if (initialize(mode))
+        return true;
 
-    if (!::GetCommState(handle, &restoredDcb)) {
-        q->setError(decodeSystemError());
-        return false;
-    }
-
-    currentDcb = restoredDcb;
-    currentDcb.fBinary = TRUE;
-    currentDcb.fInX = FALSE;
-    currentDcb.fOutX = FALSE;
-    currentDcb.fAbortOnError = FALSE;
-    currentDcb.fNull = FALSE;
-    currentDcb.fErrorChar = FALSE;
-
-    if (currentDcb.fDtrControl ==  DTR_CONTROL_HANDSHAKE)
-        currentDcb.fDtrControl = DTR_CONTROL_DISABLE;
-
-    if (!updateDcb())
-        return false;
-
-    if (!::GetCommTimeouts(handle, &restoredCommTimeouts)) {
-        q->setError(decodeSystemError());
-        return false;
-    }
-
-    ::ZeroMemory(&currentCommTimeouts, sizeof(currentCommTimeouts));
-    currentCommTimeouts.ReadIntervalTimeout = MAXDWORD;
-
-    if (!updateCommTimeouts())
-        return false;
-
-    if (mode & QIODevice::ReadOnly)
-        readCompletionNotifier->setEnabled(true);
-
-    if (mode & QIODevice::WriteOnly)
-        writeCompletionNotifier->setEnabled(true);
-
-    if (!::SetCommMask(handle, originalEventMask)) {
-        q->setError(decodeSystemError());
-        return false;
-    }
-
-    if (!startAsyncCommunication())
-        return false;
-
-    communicationNotifier->setEnabled(true);
-
-    return true;
+    ::CloseHandle(handle);
+    return false;
 }
 
 void QSerialPortPrivate::close()
@@ -243,8 +194,6 @@ void QSerialPortPrivate::close()
     handle = INVALID_HANDLE_VALUE;
 }
 
-#endif // #ifndef Q_OS_WINCE
-
 QSerialPort::PinoutSignals QSerialPortPrivate::pinoutSignals()
 {
     Q_Q(QSerialPort);
@@ -308,24 +257,9 @@ bool QSerialPortPrivate::setRequestToSend(bool set)
     return true;
 }
 
-#ifndef Q_OS_WINCE
-
 bool QSerialPortPrivate::flush()
 {
-    Q_Q(QSerialPort);
-
-    bool returnValue = true;
-
-    if (!startAsyncWrite())
-        returnValue = false;
-
-    if (!::FlushFileBuffers(handle)) {
-        q->setError(decodeSystemError());
-        returnValue = false;
-    }
-
-    return returnValue;
-
+    return _q_startAsyncWrite();
 }
 
 bool QSerialPortPrivate::clear(QSerialPort::Directions directions)
@@ -347,8 +281,6 @@ bool QSerialPortPrivate::clear(QSerialPort::Directions directions)
     return true;
 }
 
-#endif
-
 bool QSerialPortPrivate::sendBreak(int duration)
 {
     if (!setBreakEnabled(true))
@@ -374,12 +306,18 @@ bool QSerialPortPrivate::setBreakEnabled(bool set)
     return true;
 }
 
-#ifndef Q_OS_WINCE
-
 void QSerialPortPrivate::startWriting()
 {
-    if (!writeStarted)
-        startAsyncWrite();
+    Q_Q(QSerialPort);
+
+    if (!writeStarted) {
+        if (!startAsyncWriteTimer) {
+            startAsyncWriteTimer = new QTimer(q);
+            q->connect(startAsyncWriteTimer, SIGNAL(timeout()), q, SLOT(_q_startAsyncWrite()));
+            startAsyncWriteTimer->setSingleShot(true);
+        }
+        startAsyncWriteTimer->start(0);
+    }
 }
 
 bool QSerialPortPrivate::waitForReadyRead(int msecs)
@@ -389,6 +327,9 @@ bool QSerialPortPrivate::waitForReadyRead(int msecs)
     QElapsedTimer stopWatch;
     stopWatch.start();
 
+    if (!writeStarted && !_q_startAsyncWrite())
+        return false;
+
     const qint64 initialReadBufferSize = readBuffer.size();
     qint64 currentReadBufferSize = initialReadBufferSize;
 
@@ -404,15 +345,23 @@ bool QSerialPortPrivate::waitForReadyRead(int msecs)
         }
 
         if (triggeredEvent == communicationOverlapped.hEvent) {
-            _q_completeAsyncCommunication();
+            if (!_q_completeAsyncCommunication())
+                return false;
         } else if (triggeredEvent == readCompletionOverlapped.hEvent) {
-            _q_completeAsyncRead();
-            if (qint64(readBuffer.size()) != currentReadBufferSize)
+            if (!_q_completeAsyncRead())
+                return false;
+            const qint64 readBytesForOneReadOperation = qint64(readBuffer.size()) - currentReadBufferSize;
+            if (readBytesForOneReadOperation == ReadChunkSize) {
                 currentReadBufferSize = readBuffer.size();
-            else if (initialReadBufferSize != currentReadBufferSize)
+            } else if (readBytesForOneReadOperation == 0) {
+                if (initialReadBufferSize != currentReadBufferSize)
+                    return true;
+            } else {
                 return true;
+            }
         } else if (triggeredEvent == writeCompletionOverlapped.hEvent) {
-            _q_completeAsyncWrite();
+            if (!_q_completeAsyncWrite())
+                return false;
         } else {
             return false;
         }
@@ -432,6 +381,9 @@ bool QSerialPortPrivate::waitForBytesWritten(int msecs)
     QElapsedTimer stopWatch;
     stopWatch.start();
 
+    if (!writeStarted && !_q_startAsyncWrite())
+        return false;
+
     forever {
         bool timedOut = false;
         HANDLE triggeredEvent = 0;
@@ -443,12 +395,13 @@ bool QSerialPortPrivate::waitForBytesWritten(int msecs)
         }
 
         if (triggeredEvent == communicationOverlapped.hEvent) {
-             _q_completeAsyncRead();
+            if (!_q_completeAsyncCommunication())
+                return false;
         } else if (triggeredEvent == readCompletionOverlapped.hEvent) {
-            _q_completeAsyncRead();
+            if (!_q_completeAsyncRead())
+                return false;
         } else if (triggeredEvent == writeCompletionOverlapped.hEvent) {
-            _q_completeAsyncWrite();
-            return error == QSerialPort::NoError;
+            return _q_completeAsyncWrite();
         } else {
             return false;
         }
@@ -458,8 +411,6 @@ bool QSerialPortPrivate::waitForBytesWritten(int msecs)
     return false;
 }
 
-#endif // #ifndef Q_OS_WINCE
-
 bool QSerialPortPrivate::setBaudRate()
 {
     return setBaudRate(inputBaudRate, QSerialPort::AllDirections);
@@ -559,80 +510,51 @@ bool QSerialPortPrivate::setDataErrorPolicy(QSerialPort::DataErrorPolicy policy)
     return true;
 }
 
-#ifndef Q_OS_WINCE
-
-void QSerialPortPrivate::_q_completeAsyncCommunication()
+bool QSerialPortPrivate::_q_completeAsyncCommunication()
 {
-    Q_Q(QSerialPort);
+    if (handleOverlappedResult(0, communicationOverlapped) == qint64(-1))
+        return false;
+    if (EV_ERR & triggeredEventMask)
+        handleLineStatusErrors();
 
-    DWORD numberOfBytesTransferred = 0;
-
-    if (!::GetOverlappedResult(handle, &communicationOverlapped, &numberOfBytesTransferred, FALSE))
-        q->setError(decodeSystemError());
-
-    bool error = false;
-
-    // Check for unexpected event. This event triggered when pulled previously
-    // opened device from the system, when opened as for not to read and not to
-    // write options and so forth.
-    if (triggeredEventMask == 0)
-        error = true;
-
-    // Workaround for standard CDC ACM serial ports, for which triggered an
-    // unexpected event EV_TXEMPTY at data transmission.
-    if ((originalEventMask & triggeredEventMask) == 0) {
-        if ((triggeredEventMask & EV_TXEMPTY) == 0)
-            error = true;
-    }
-
-    // Start processing a caught error.
-    if (error || (EV_ERR & triggeredEventMask))
-        processIoErrors(error);
-
-    if (!error)
-        startAsyncRead();
+    return startAsyncRead();
 }
 
-void QSerialPortPrivate::_q_completeAsyncRead()
+bool QSerialPortPrivate::_q_completeAsyncRead()
 {
-    Q_Q(QSerialPort);
-
-    DWORD numberOfBytesTransferred = 0;
-    if (!::GetOverlappedResult(handle, &readCompletionOverlapped, &numberOfBytesTransferred, FALSE))
-        q->setError(decodeSystemError());
-
-    if (numberOfBytesTransferred > 0) {
-
-        readBuffer.append(readChunkBuffer.left(numberOfBytesTransferred));
-
+    const qint64 bytesTransferred = handleOverlappedResult(QSerialPort::Input, readCompletionOverlapped);
+    if (bytesTransferred == qint64(-1))
+        return false;
+    if (bytesTransferred > 0) {
+        readBuffer.append(readChunkBuffer.left(bytesTransferred));
         if (!emulateErrorPolicy())
             emitReadyRead();
     }
 
     // start async read for possible remainder into driver queue
-    if ((numberOfBytesTransferred > 0) && (policy == QSerialPort::IgnorePolicy))
-        startAsyncRead();
+    if ((bytesTransferred == ReadChunkSize) && (policy == QSerialPort::IgnorePolicy))
+        return startAsyncRead();
     else // driver queue is emplty, so startup wait comm event
-        startAsyncCommunication();
+        return startAsyncCommunication();
 }
 
-void QSerialPortPrivate::_q_completeAsyncWrite()
+bool QSerialPortPrivate::_q_completeAsyncWrite()
 {
     Q_Q(QSerialPort);
 
-    DWORD numberOfBytesTransferred = 0;
-    if (!::GetOverlappedResult(handle, &writeCompletionOverlapped, &numberOfBytesTransferred, FALSE)) {
-        numberOfBytesTransferred = 0;
-        q->setError(decodeSystemError());
+    if (writeStarted) {
+        const qint64 bytesTransferred = handleOverlappedResult(QSerialPort::Output, writeCompletionOverlapped);
+        if (bytesTransferred == qint64(-1)) {
+            writeStarted = false;
+            return false;
+        } else if (bytesTransferred > 0) {
+            writeBuffer.free(bytesTransferred);
+            emit q->bytesWritten(bytesTransferred);
+        }
+        writeStarted = false;
     }
 
-    if (numberOfBytesTransferred > 0) {
-        writeBuffer.free(numberOfBytesTransferred);
-        emit q->bytesWritten(numberOfBytesTransferred);
-    }
-
-    writeStarted = false;
-    startAsyncWrite();
+    return _q_startAsyncWrite();
 }
 
 bool QSerialPortPrivate::startAsyncCommunication()
@@ -641,9 +563,11 @@ bool QSerialPortPrivate::startAsyncCommunication()
 
     initializeOverlappedStructure(communicationOverlapped);
     if (!::WaitCommEvent(handle, &triggeredEventMask, &communicationOverlapped)) {
-        const QSerialPort::SerialPortError error = decodeSystemError();
+        QSerialPort::SerialPortError error = decodeSystemError();
         if (error != QSerialPort::NoError) {
-            q->setError(decodeSystemError());
+            if (error == QSerialPort::PermissionError)
+                error = QSerialPort::ResourceError;
+            q->setError(error);
             return false;
         }
     }
@@ -671,17 +595,18 @@ bool QSerialPortPrivate::startAsyncRead()
 
     QSerialPort::SerialPortError error = decodeSystemError();
     if (error != QSerialPort::NoError) {
+        if (error == QSerialPort::PermissionError)
+            error = QSerialPort::ResourceError;
         if (error != QSerialPort::ResourceError)
             error = QSerialPort::ReadError;
         q->setError(error);
-
         return false;
     }
 
     return true;
 }
 
-bool QSerialPortPrivate::startAsyncWrite()
+bool QSerialPortPrivate::_q_startAsyncWrite()
 {
     Q_Q(QSerialPort);
 
@@ -742,17 +667,10 @@ void QSerialPortPrivate::emitReadyRead()
     emit q->readyRead();
 }
 
-#endif // #ifndef Q_OS_WINCE
-
-void QSerialPortPrivate::processIoErrors(bool error)
+void QSerialPortPrivate::handleLineStatusErrors()
 {
     Q_Q(QSerialPort);
 
-    if (error) {
-        q->setError(QSerialPort::ResourceError);
-        return;
-    }
-
     DWORD errors = 0;
     if (!::ClearCommError(handle, &errors, NULL)) {
         q->setError(decodeSystemError());
@@ -771,7 +689,61 @@ void QSerialPortPrivate::processIoErrors(bool error)
     }
 }
 
-#ifndef Q_OS_WINCE
+inline bool QSerialPortPrivate::initialize(QIODevice::OpenMode mode)
+{
+    Q_Q(QSerialPort);
+
+    ::ZeroMemory(&restoredDcb, sizeof(restoredDcb));
+    restoredDcb.DCBlength = sizeof(restoredDcb);
+
+    if (!::GetCommState(handle, &restoredDcb)) {
+        q->setError(decodeSystemError());
+        return false;
+    }
+
+    currentDcb = restoredDcb;
+    currentDcb.fBinary = TRUE;
+    currentDcb.fInX = FALSE;
+    currentDcb.fOutX = FALSE;
+    currentDcb.fAbortOnError = FALSE;
+    currentDcb.fNull = FALSE;
+    currentDcb.fErrorChar = FALSE;
+
+    if (currentDcb.fDtrControl ==  DTR_CONTROL_HANDSHAKE)
+        currentDcb.fDtrControl = DTR_CONTROL_DISABLE;
+
+    if (!updateDcb())
+        return false;
+
+    if (!::GetCommTimeouts(handle, &restoredCommTimeouts)) {
+        q->setError(decodeSystemError());
+        return false;
+    }
+
+    ::ZeroMemory(&currentCommTimeouts, sizeof(currentCommTimeouts));
+    currentCommTimeouts.ReadIntervalTimeout = MAXDWORD;
+
+    if (!updateCommTimeouts())
+        return false;
+
+    if (mode & QIODevice::ReadOnly)
+        readCompletionNotifier->setEnabled(true);
+
+    if (mode & QIODevice::WriteOnly)
+        writeCompletionNotifier->setEnabled(true);
+
+    if (!::SetCommMask(handle, originalEventMask)) {
+        q->setError(decodeSystemError());
+        return false;
+    }
+
+    if (!startAsyncCommunication())
+        return false;
+
+    communicationNotifier->setEnabled(true);
+
+    return true;
+}
 
 bool QSerialPortPrivate::updateDcb()
 {
@@ -795,7 +767,27 @@ bool QSerialPortPrivate::updateCommTimeouts()
     return true;
 }
 
-#endif // #ifndef Q_OS_WINCE
+qint64 QSerialPortPrivate::handleOverlappedResult(int direction, OVERLAPPED &overlapped)
+{
+    Q_Q(QSerialPort);
+
+    DWORD bytesTransferred = 0;
+    if (!::GetOverlappedResult(handle, &overlapped, &bytesTransferred, FALSE)) {
+        const QSerialPort::SerialPortError error = decodeSystemError();
+        if (error == QSerialPort::NoError)
+            return qint64(0);
+        if (error != QSerialPort::ResourceError) {
+            if (direction == QSerialPort::Input)
+                q->setError(QSerialPort::ReadError);
+            else if (direction == QSerialPort::Output)
+                q->setError(QSerialPort::WriteError);
+            else
+                q->setError(error);
+            return qint64(-1);
+        }
+    }
+    return bytesTransferred;
+}
 
 QSerialPort::SerialPortError QSerialPortPrivate::decodeSystemError() const
 {
@@ -828,6 +820,9 @@ QSerialPort::SerialPortError QSerialPortPrivate::decodeSystemError() const
     case ERROR_DEVICE_REMOVED:
         error = QSerialPort::ResourceError;
         break;
+    case ERROR_OPERATION_ABORTED:
+        error = QSerialPort::ResourceError;
+        break;
     default:
         error = QSerialPort::UnknownError;
         break;
@@ -835,8 +830,6 @@ QSerialPort::SerialPortError QSerialPortPrivate::decodeSystemError() const
     return error;
 }
 
-#ifndef Q_OS_WINCE
-
 bool QSerialPortPrivate::waitAnyEvent(int msecs, bool *timedOut, HANDLE *triggeredEvent)
 {
     Q_Q(QSerialPort);
@@ -854,7 +847,7 @@ bool QSerialPortPrivate::waitAnyEvent(int msecs, bool *timedOut, HANDLE *trigger
                                                 msecs == -1 ? INFINITE : msecs);
     if (waitResult == WAIT_TIMEOUT) {
         *timedOut = true;
-        q->setError(QSerialPort::TimeoutError);
+        q->setError(QSerialPort::TimeoutError, qt_error_string(WAIT_TIMEOUT));
         return false;
     }
 
@@ -883,8 +876,6 @@ QString QSerialPortPrivate::portNameFromSystemLocation(const QString &location)
     return ret;
 }
 
-#endif // #ifndef Q_OS_WINCE
-
 // This table contains standard values of baud rates that
 // are defined in MSDN and/or in Win SDK file winbase.h
 
