@@ -42,6 +42,17 @@
 #ifndef QTUDEV_P_H
 #define QTUDEV_P_H
 
+//
+//  W A R N I N G
+//  -------------
+//
+// This file is not part of the Qt API.  It exists purely as an
+// implementation detail.  This header file may change from version to
+// version without notice, or even be removed.
+//
+// We mean it.
+//
+
 #ifdef LINK_LIBUDEV
 extern "C"
 {
@@ -82,6 +93,7 @@ GENERATE_SYMBOL_VARIABLE(struct udev_device *, udev_device_new_from_syspath, str
 GENERATE_SYMBOL_VARIABLE(const char *, udev_list_entry_get_name, struct udev_list_entry *)
 GENERATE_SYMBOL_VARIABLE(const char *, udev_device_get_devnode, struct udev_device *)
 GENERATE_SYMBOL_VARIABLE(const char *, udev_device_get_sysname, struct udev_device *)
+GENERATE_SYMBOL_VARIABLE(const char *, udev_device_get_driver, struct udev_device *)
 GENERATE_SYMBOL_VARIABLE(struct udev_device *, udev_device_get_parent, struct udev_device *)
 GENERATE_SYMBOL_VARIABLE(const char *, udev_device_get_subsystem, struct udev_device *)
 GENERATE_SYMBOL_VARIABLE(const char *, udev_device_get_property_value, struct udev_device *, const char *)
@@ -127,6 +139,7 @@ inline bool resolveSymbols(QLibrary *udevLibrary)
     RESOLVE_SYMBOL(udev_list_entry_get_name)
     RESOLVE_SYMBOL(udev_device_get_devnode)
     RESOLVE_SYMBOL(udev_device_get_sysname)
+    RESOLVE_SYMBOL(udev_device_get_driver)
     RESOLVE_SYMBOL(udev_device_get_parent)
     RESOLVE_SYMBOL(udev_device_get_subsystem)
     RESOLVE_SYMBOL(udev_device_get_property_value)
